MODULE agrif_oce
   !!======================================================================
   !!                       ***  MODULE agrif_oce  ***
   !! AGRIF :   define in memory AGRIF variables
   !!----------------------------------------------------------------------
   !! History :  2.0  ! 2007-12  (R. Benshila)  Original code
   !!----------------------------------------------------------------------
#if defined key_agrif
   !!----------------------------------------------------------------------
   !!   'key_agrif'                                              AGRIF zoom
   !!----------------------------------------------------------------------
   USE par_oce      ! ocean parameters
   USE dom_oce      ! domain parameters

   IMPLICIT NONE
   PRIVATE 

   PUBLIC agrif_oce_alloc ! routine called by nemo_init in nemogcm.F90
   PUBLIC reconstructandremap

   !                                              !!* Namelist namagrif: AGRIF parameters
   LOGICAL , PUBLIC ::   ln_spc_dyn    = .FALSE.   !:
   INTEGER , PUBLIC ::   nn_cln_update = 3         !: update frequency 
   INTEGER , PUBLIC, PARAMETER ::   nn_sponge_len = 2  !: Sponge width (in number of parent grid points)
   REAL(wp), PUBLIC ::   rn_sponge_tra = 2800.     !: sponge coeff. for tracers
   REAL(wp), PUBLIC ::   rn_sponge_dyn = 2800.     !: sponge coeff. for dynamics
   LOGICAL , PUBLIC ::   ln_chk_bathy  = .FALSE.   !: check of parent bathymetry 

   !                                              !!! OLD namelist names
   INTEGER , PUBLIC ::   nbcline = 0               !: update counter
   INTEGER , PUBLIC ::   nbclineupdate             !: update frequency 
   REAL(wp), PUBLIC ::   visc_tra                  !: sponge coeff. for tracers
   REAL(wp), PUBLIC ::   visc_dyn                  !: sponge coeff. for dynamics

   LOGICAL , PUBLIC :: spongedoneT = .FALSE.       !: tracer   sponge layer indicator
   LOGICAL , PUBLIC :: spongedoneU = .FALSE.       !: dynamics sponge layer indicator
   LOGICAL , PUBLIC :: lk_agrif_fstep = .TRUE.     !: if true: first step
   LOGICAL , PUBLIC :: lk_agrif_doupd = .TRUE.     !: if true: send update from current grid
   LOGICAL , PUBLIC :: lk_agrif_debug = .FALSE.    !: if true: print debugging info

   LOGICAL , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: tabspongedone_tsn
# if defined key_top
   LOGICAL , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: tabspongedone_trn
# endif
   LOGICAL , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: tabspongedone_u
   LOGICAL , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) :: tabspongedone_v
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE,  DIMENSION(:,:) :: fsaht_spu, fsaht_spv !: sponge diffusivities
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE,  DIMENSION(:,:) :: fsahm_spt, fsahm_spf !: sponge viscosities

   ! Barotropic arrays used to store open boundary data during
   ! time-splitting loop:
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:) ::  ubdy_w, vbdy_w, hbdy_w
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:) ::  ubdy_e, vbdy_e, hbdy_e
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:) ::  ubdy_n, vbdy_n, hbdy_n
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:) ::  ubdy_s, vbdy_s, hbdy_s

   INTEGER :: tsn_id                                                  ! AGRIF profile for tracers interpolation and update
   INTEGER :: un_interp_id, vn_interp_id                              ! AGRIF profiles for interpolations
   INTEGER :: un_update_id, vn_update_id                              ! AGRIF profiles for udpates
   INTEGER :: tsn_sponge_id, un_sponge_id, vn_sponge_id               ! AGRIF profiles for sponge layers
# if defined key_top
   INTEGER :: trn_id, trn_sponge_id
# endif  
   INTEGER :: unb_id, vnb_id, ub2b_interp_id, vb2b_interp_id
   INTEGER :: ub2b_update_id, vb2b_update_id
   INTEGER :: e3t_id, e1u_id, e2v_id, sshn_id
   INTEGER :: scales_t_id
# if defined key_zdftke
   INTEGER :: avt_id, avm_id, en_id
# endif  
   INTEGER :: umsk_id, vmsk_id
   INTEGER :: kindic_agr

   !!----------------------------------------------------------------------
   !! NEMO/NST 3.3.1 , NEMO Consortium (2011)
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS 

   INTEGER FUNCTION agrif_oce_alloc()
      !!----------------------------------------------------------------------
      !!                ***  FUNCTION agrif_oce_alloc  ***
      !!----------------------------------------------------------------------
      INTEGER, DIMENSION(2) :: ierr
      !!----------------------------------------------------------------------
      ierr(:) = 0
      !
      ALLOCATE( fsaht_spu(jpi,jpj), fsaht_spv(jpi,jpj),   &
         &      fsahm_spt(jpi,jpj), fsahm_spf(jpi,jpj),   &
         &      tabspongedone_tsn(jpi,jpj),           &
# if defined key_top         
         &      tabspongedone_trn(jpi,jpj),           &
# endif         
         &      tabspongedone_u  (jpi,jpj),           &
         &      tabspongedone_v  (jpi,jpj), STAT = ierr(1) )

      ALLOCATE( ubdy_w(jpj), vbdy_w(jpj), hbdy_w(jpj),   &
         &      ubdy_e(jpj), vbdy_e(jpj), hbdy_e(jpj),   & 
         &      ubdy_n(jpi), vbdy_n(jpi), hbdy_n(jpi),   & 
         &      ubdy_s(jpi), vbdy_s(jpi), hbdy_s(jpi), STAT = ierr(2) )

      agrif_oce_alloc = MAXVAL(ierr)
      !
   END FUNCTION agrif_oce_alloc

      subroutine reconstructandremap(tabin,hin,tabout,hout,N,Nout)      
      implicit none
      integer N, Nout
      real tabin(N), tabout(Nout)
      real hin(N), hout(Nout)
      real coeffremap(N,3),zwork(N,3)
      real zwork2(N+1,3)
      integer k
      double precision, parameter :: dsmll=1.0d-8  
      real q,q01,q02,q001,q002,q0
      real z_win(1:N+1), z_wout(1:Nout+1)
      real,parameter :: dpthin = 1.D-3
      integer :: k1, kbox, ktop, ka, kbot
      real :: tsum, qbot, rpsum, zbox, ztop, zthk, zbot, offset, qtop

      z_win(1)=0.; z_wout(1)= 0.
      do k=1,N
       z_win(k+1)=z_win(k)+hin(k)
      enddo 
      
      do k=1,Nout
       z_wout(k+1)=z_wout(k)+hout(k)       
      enddo       

        do k=2,N
          zwork(k,1)=1./(hin(k-1)+hin(k))
        enddo
        
        do k=2,N-1
          q0 = 1./(hin(k-1)+hin(k)+hin(k+1))
          zwork(k,2)=hin(k-1)+2.*hin(k)+hin(k+1)
          zwork(k,3)=q0
        enddo       
      
        do k= 2,N
        zwork2(k,1)=zwork(k,1)*(tabin(k)-tabin(k-1))
        enddo
        
        coeffremap(:,1) = tabin(:)
 
         do k=2,N-1
        q001 = hin(k)*zwork2(k+1,1)
        q002 = hin(k)*zwork2(k,1)        
        if (q001*q002 < 0) then
          q001 = 0.
          q002 = 0.
        endif
        q=zwork(k,2)
        q01=q*zwork2(k+1,1)
        q02=q*zwork2(k,1)
        if (abs(q001) > abs(q02)) q001 = q02
        if (abs(q002) > abs(q01)) q002 = q01
        
        q=(q001-q002)*zwork(k,3)
        q001=q001-q*hin(k+1)
        q002=q002+q*hin(k-1)
        
        coeffremap(k,3)=coeffremap(k,1)+q001
        coeffremap(k,2)=coeffremap(k,1)-q002
        
        zwork2(k,1)=(2.*q001-q002)**2
        zwork2(k,2)=(2.*q002-q001)**2
        enddo
        
        do k=1,N
        if     (k.eq.1 .or. k.eq.N .or.   hin(k).le.dpthin) then
        coeffremap(k,3) = coeffremap(k,1)
        coeffremap(k,2) = coeffremap(k,1)
        zwork2(k,1) = 0.
        zwork2(k,2) = 0.
        endif
        enddo
        
        do k=2,N
        q002=max(zwork2(k-1,2),dsmll)
        q001=max(zwork2(k,1),dsmll)
        zwork2(k,3)=(q001*coeffremap(k-1,3)+q002*coeffremap(k,2))/(q001+q002)
        enddo
        
        zwork2(1,3) = 2*coeffremap(1,1)-zwork2(2,3)
        zwork2(N+1,3)=2*coeffremap(N,1)-zwork2(N,3)
 
        do k=1,N
        q01=zwork2(k+1,3)-coeffremap(k,1)
        q02=coeffremap(k,1)-zwork2(k,3)
        q001=2.*q01
        q002=2.*q02
        if (q01*q02<0) then
          q01=0.
          q02=0.
        elseif (abs(q01)>abs(q002)) then
          q01=q002
        elseif (abs(q02)>abs(q001)) then
          q02=q001
        endif
        coeffremap(k,2)=coeffremap(k,1)-q02
        coeffremap(k,3)=coeffremap(k,1)+q01
        enddo

      zbot=0.0
      kbot=1
      do k=1,Nout
        ztop=zbot  !top is bottom of previous layer
        ktop=kbot
        if     (ztop.ge.z_win(ktop+1)) then
          ktop=ktop+1
        endif
        
        zbot=z_wout(k+1)
        zthk=zbot-ztop

        if     (zthk.gt.dpthin .and. ztop.lt.z_wout(Nout+1)) then

          kbot=ktop
          do while (z_win(kbot+1).lt.zbot.and.kbot.lt.N)
            kbot=kbot+1
          enddo
          zbox=zbot
          do k1= k+1,Nout
            if     (z_wout(k1+1)-z_wout(k1).gt.dpthin) then
              exit !thick layer
            else
              zbox=z_wout(k1+1)  !include thin adjacent layers
              if     (zbox.eq.z_wout(Nout+1)) then
                exit !at bottom
              endif
            endif
          enddo
          zthk=zbox-ztop

          kbox=ktop
          do while (z_win(kbox+1).lt.zbox.and.kbox.lt.N)
            kbox=kbox+1
          enddo
          
          if     (ktop.eq.kbox) then


            if     (z_wout(k)  .ne.z_win(kbox)   .or.z_wout(k+1).ne.z_win(kbox+1)     ) then

              if     (hin(kbox).gt.dpthin) then
                q001 = (zbox-z_win(kbox))/hin(kbox)
                q002 = (ztop-z_win(kbox))/hin(kbox)
                q01=q001**2+q002**2+q001*q002+1.-2.*(q001+q002)
                q02=q01-1.+(q001+q002)
                q0=1.-q01-q02
              else
                q0 = 1.0
                q01 = 0.
                q02 = 0.
              endif
          tabout(k)=q0*coeffremap(kbox,1)+q01*coeffremap(kbox,2)+q02*coeffremap(kbox,3)
              
            else
            tabout(k) = tabin(kbox)
              
            endif 

          else

            if     (ktop.le.k .and. kbox.ge.k) then
              ka = k
            elseif (kbox-ktop.ge.3) then
              ka = (kbox+ktop)/2
            elseif (hin(ktop).ge.hin(kbox)) then
              ka = ktop
            else
              ka = kbox
            endif !choose ka

            offset=coeffremap(ka,1)

            qtop = z_win(ktop+1)-ztop !partial layer thickness
            if     (hin(ktop).gt.dpthin) then
              q=(ztop-z_win(ktop))/hin(ktop)
              q01=q*(q-1.)
              q02=q01+q
              q0=1-q01-q02            
            else
              q0 = 1.
              q01 = 0.
              q02 = 0.
            endif
            
            tsum =((q0*coeffremap(ktop,1)+q01*coeffremap(ktop,2)+q02*coeffremap(ktop,3))-offset)*qtop

            do k1= ktop+1,kbox-1
              tsum =tsum +(coeffremap(k1,1)-offset)*hin(k1)
            enddo !k1

            
            qbot = zbox-z_win(kbox) !partial layer thickness
            if     (hin(kbox).gt.dpthin) then
              q=qbot/hin(kbox)
              q01=(q-1.)**2
              q02=q01-1.+q
              q0=1-q01-q02                            
            else
              q0 = 1.0
              q01 = 0.
              q02 = 0.
            endif
           
            tsum = tsum +((q0*coeffremap(kbox,1)+q01*coeffremap(kbox,2)+q02*coeffremap(kbox,3))-offset)*qbot
            
            rpsum=1.0d0/zthk
              tabout(k)=offset+tsum*rpsum
              
          endif !single or multiple layers
        else
        if (k==1) then
        write(*,'(a7,i4,i4,3f12.5)')'problem = ',N,Nout,zthk,z_wout(k+1),hout(1)
        endif
         tabout(k) = tabout(k-1)
          
        endif !normal:thin layer
      enddo !k
            
      return
      end subroutine reconstructandremap
      
#endif
   !!======================================================================
END MODULE agrif_oce
